from django.apps import AppConfig

from django.conf import settings


class DseagullConfig(AppConfig):
    name = 'dseagull'
    verbose_name = "dseagull"

    def ready(self):
        from .checks import jwt_check  # noqa
        self.default_rest_framework()

    @classmethod
    def default_rest_framework(cls):

        if not hasattr(settings, 'REST_FRAMEWORK'):
            settings.REST_FRAMEWORK = {}

        # 指定默认分页类
        if 'DEFAULT_PAGINATION_CLASS' not in settings.REST_FRAMEWORK:
            settings.REST_FRAMEWORK['DEFAULT_PAGINATION_CLASS'] = 'dseagull.pagination.PageNumberPagination'

        # 默认每页 10 条数据
        if 'PAGE_SIZE' not in settings.REST_FRAMEWORK:
            settings.REST_FRAMEWORK['PAGE_SIZE'] = 10

        # 默认 OpenAPI 规范文档类为 rest_framework.schemas.coreapi.AutoSchema
        if 'DEFAULT_SCHEMA_CLASS' not in settings.REST_FRAMEWORK:
            settings.REST_FRAMEWORK['DEFAULT_SCHEMA_CLASS'] = 'rest_framework.schemas.coreapi.AutoSchema'

        # 默认指定 DEFAULT_FILTER_BACKENDS
        if 'DEFAULT_FILTER_BACKENDS' not in settings.REST_FRAMEWORK:
            settings.REST_FRAMEWORK['DEFAULT_FILTER_BACKENDS'] = [
                'django_filters.rest_framework.DjangoFilterBackend',
                'rest_framework.filters.SearchFilter',
                'rest_framework.filters.OrderingFilter',
            ]

        # 默认指定 TEST_REQUEST_DEFAULT_FORMAT
        if 'TEST_REQUEST_DEFAULT_FORMAT' not in settings.REST_FRAMEWORK:
            settings.REST_FRAMEWORK['TEST_REQUEST_DEFAULT_FORMAT'] = 'json'

        # 默认指定 DEFAULT_RENDERER_CLASSES
        if 'DEFAULT_RENDERER_CLASSES' not in settings.REST_FRAMEWORK:
            settings.REST_FRAMEWORK['DEFAULT_RENDERER_CLASSES'] = ('rest_framework.renderers.JSONRenderer',)



from rest_framework.serializers import Serializer

from dseagull.serializers import BooleanField
from dseagull.serializers import CharField
from dseagull.serializers import DateField
from dseagull.serializers import DateTimeField
from dseagull.serializers import DictField
from dseagull.serializers import DurationField
from dseagull.serializers import EmailField
from dseagull.serializers import Field
from dseagull.serializers import FloatField
from dseagull.serializers import HStoreField
from dseagull.serializers import IPAddressField
from dseagull.serializers import ImageField
from dseagull.serializers import IntegerField
from dseagull.serializers import JSONField
from dseagull.serializers import ListField
from dseagull.serializers import SlugField
from dseagull.serializers import TimeField
from dseagull.serializers import URLField
from dseagull.serializers import UUIDField


class TestFieldErrorMessages:
    def test_required(self):
        for field in (BooleanField, CharField, DateField, DateTimeField, DictField, DurationField,
                      EmailField, Field, FloatField, HStoreField, IPAddressField, ImageField,
                      IntegerField, JSONField, ListField, SlugField,
                      TimeField, URLField, UUIDField):
            class ExampleSerializer(Serializer):
                name = field(help_text='姓名')

            serializer = ExampleSerializer(data={})
            serializer.is_valid()
            assert serializer.errors['name'][0] == '姓名:这个字段是必填项。', field

        class ExampleSerializer(Serializer):
            name = field(help_text='姓名', error_messages={'required': '请填入姓名。'})

        serializer = ExampleSerializer(data={})
        serializer.is_valid()
        assert serializer.errors['name'][0] == '请填入姓名。'

        class ExampleSerializer(Serializer):
            name = field(help_text='姓名(string)', error_help_text="姓名A")

        serializer = ExampleSerializer(data={})
        serializer.is_valid()
        assert serializer.errors['name'][0] == '姓名A:这个字段是必填项。'

        class ExampleSerializer(Serializer):
            name = field(error_help_text="姓名B")

        serializer = ExampleSerializer(data={})
        serializer.is_valid()
        assert serializer.errors['name'][0] == '姓名B:这个字段是必填项。'

    def test_null(self):
        class ExampleSerializer(Serializer):
            name = CharField(help_text='姓名')

        serializer = ExampleSerializer(data={'name': None})
        serializer.is_valid()
        assert serializer.errors['name'][0] == '姓名:不能为空。'

    def test_blank(self):
        class ExampleSerializer(Serializer):
            name = CharField(help_text='姓名')

        serializer = ExampleSerializer(data={'name': ''})
        serializer.is_valid()
        assert serializer.errors['name'][0] == '姓名:不能为空白。'



import pytest
from django.test import RequestFactory, TestCase
from rest_framework import (
    serializers
)
from rest_framework.viewsets import ModelViewSet

from dseagull.logger import thread_local
from dseagull.middleware import BaseMiddleware
from tests.models.modeltest import Person


class PersonSerializer(serializers.ModelSerializer):
    first_name = serializers.CharField()

    class Meta:
        model = Person
        fields = ('id', 'first_name',)


class PersonViewSet(ModelViewSet):
    serializer_class = PersonSerializer
    queryset = Person.objects.all()


class TestMiddleware(TestCase):
    #
    def setUp(self):
        Person.objects.create(first_name='name1', last_name='last_name1')
        Person.objects.create(first_name='name2', last_name='last_name2')

    @pytest.mark.django_db
    def test_base_middleware(self):
        thread_local.remote_ip = 'remote_ip'
        factory = RequestFactory()
        factory.META = factory.GET = {'HTTP_AUTHORIZATION': f'Bearer x'}
        BaseMiddleware(get_response=lambda x: None)(factory)  # noqa
        factory.content = factory.body = b'{}'
        factory.path = '/'
        factory.method = 'POST'
        BaseMiddleware(get_response=lambda x: None)(factory)  # noqa



import jwt
from django.conf import settings
from django.test import TestCase, override_settings

from dseagull.djwt import JWTHS256, JWTUser


class TestDjwt(TestCase):

    def test_encode_decode(self):
        """ 正常编码解码 """
        settings.JWT_EXP = 60
        jwt_obj = JWTHS256()
        token = jwt_obj.encode({'username': 'admin'})
        payload = jwt_obj.decode(token)
        payload.pop('exp')
        self.assertEqual(payload, {'username': 'admin'})

    def test_encode_decode_invalid_signature(self):
        """ 密钥错误测试 """
        jwt_obj = JWTHS256()
        token = jwt_obj.encode({'username': 'admin'})

        with self.assertRaises(jwt.InvalidSignatureError):
            jwt_obj.key = 'd6b15627964c43b290d803e0f4851d13'
            jwt_obj.decode(token)

    @override_settings(JWT_EXP=-61)
    def test_encode_decode_expired_signature(self):
        """ 过期测试 """
        jwt_obj = JWTHS256()
        token = jwt_obj.encode({'username': 'admin'})
        with self.assertRaises(jwt.ExpiredSignatureError):
            jwt_obj.decode(token)

    def test_jwtuser(self):
        JWTUser(user_id=1, user_type='user_type', )



import logging
import traceback

import requests
from django.conf import settings


class DjangoRequestErrorLOGGINGHandler(logging.Handler):

    def emit(self, record):

        webhook = getattr(settings, 'DJANGO_REQUEST_ERROR_WEBHOOK', None)
        if not webhook:
            return

        msg = record.msg or ""
        if msg.startswith("Task %") or msg.startswith("%s: %s"):
            record.msg = traceback.format_exc().replace('\n', '\n\n')

        payload = {
            "msgtype": "markdown",
            "markdown": {
                "title": "Seagull",
                "text": record.msg,
            }
        }
        # 钉钉文档: https://open.dingtalk.com/document/orgapp/custom-robot-access
        r = requests.post(webhook, json=payload, timeout=10)
        if r.status_code == 200 and b'"errmsg":"ok"' in r.content:
            pass
        else:
            print(f"[DjangoRequestErrorLOGGINGHandler] {r.content.decode()}")


LOGGER = logging.getLogger('django.request')



import time

import jwt
from django.conf import settings


class JWTHS256:
    """ 使用对称加密:HS256 """

    def __init__(self, ):
        self.key = settings.JWT_KEY  # 密钥
        self.exp = settings.JWT_EXP

    def encode(self, payload: dict) -> str:
        """ 编码 """
        payload["exp"] = time.time() + self.exp
        token = jwt.encode(
            payload=payload,
            key=self.key,
            algorithm='HS256',
            headers={'alg': 'HS256', 'typ': 'JWT', },
        )
        return token

    def decode(self, jwt_data: str) -> dict:
        """ 解码 """
        payload = jwt.decode(
            jwt_data,
            self.key,
            verify=True,
            algorithms='HS256',
            leeway=60,  # 每个服务器时间不一样, 允许误差60秒
        )
        return payload


class JWTUser:

    def __init__(self, user_id: int, user_type: str, headers: dict = {}, *args, **kwargs):  # noqa
        self.user_id = user_id
        self.user_type = user_type
        self.headers = headers



from django.conf import settings
from rest_framework.serializers import BooleanField as DRFBooleanField
from rest_framework.serializers import CharField as DRFCharField
from rest_framework.serializers import ChoiceField as DRFChoiceField
from rest_framework.serializers import DateField as DRFDateField
from rest_framework.serializers import DateTimeField as DRFDateTimeField
from rest_framework.serializers import DecimalField as DRFDecimalField
from rest_framework.serializers import DictField as DRFDictField
from rest_framework.serializers import DurationField as DRFDurationField
from rest_framework.serializers import EmailField as DRFEmailField
from rest_framework.serializers import Field as DRFField
from rest_framework.serializers import FileField as DRFFileField
from rest_framework.serializers import FilePathField as DRFFilePathField
from rest_framework.serializers import FloatField as DRFFloatField
from rest_framework.serializers import HStoreField as DRFHStoreField
from rest_framework.serializers import HiddenField as DRFHiddenField
from rest_framework.serializers import IPAddressField as DRFIPAddressField
from rest_framework.serializers import ImageField as DRFImageField
from rest_framework.serializers import IntegerField as DRFIntegerField
from rest_framework.serializers import JSONField as DRFJSONField
from rest_framework.serializers import ListField as DRFListField
from rest_framework.serializers import ModelField as DRFModelField
from rest_framework.serializers import MultipleChoiceField as DRFMultipleChoiceField
from rest_framework.serializers import ReadOnlyField as DRFReadOnlyField
from rest_framework.serializers import RegexField as DRFRegexField
from rest_framework.serializers import SerializerMethodField as DRFSerializerMethodField
from rest_framework.serializers import SlugField as DRFSlugField
from rest_framework.serializers import TimeField as DRFTimeField
from rest_framework.serializers import URLField as DRFURLField
from rest_framework.serializers import UUIDField as DRFUUIDField


class Field(DRFField):

    def __init__(self, error_help_text='', *args, **kwargs):

        if "error_help_text" in kwargs and not kwargs.get('help_text'):
            kwargs["help_text"] = kwargs["error_help_text"]
        super().__init__(*args, **kwargs)
        kwargs["error_help_text"] = error_help_text
        if settings.LANGUAGE_CODE == 'zh-hans':
            if 'error_messages' not in kwargs:
                help_text = kwargs.get('error_help_text') or ''
                if not help_text:
                    help_text = kwargs.get('help_text') or ''
                help_text = f'{help_text}:' if help_text else help_text
                for key in self.error_messages:
                    if key == 'null':
                        self.error_messages['null'] = f'{help_text}不能为空。'
                    elif key == 'blank':
                        self.error_messages['blank'] = f'{help_text}不能为空白。'
                    else:
                        self.error_messages[key] = f'{help_text}{self.error_messages[key]}'


class BooleanField(Field, DRFBooleanField): pass


class CharField(Field, DRFCharField): pass


class ChoiceField(Field, DRFChoiceField): pass


class DateField(Field, DRFDateField): pass


class DateTimeField(Field, DRFDateTimeField): pass


class DecimalField(Field, DRFDecimalField): pass


class DictField(Field, DRFDictField): pass


class DurationField(Field, DRFDurationField): pass


class EmailField(Field, DRFEmailField): pass


class Field(Field, DRFField): pass


class FileField(Field, DRFFileField): pass


class FilePathField(Field, DRFFilePathField): pass


class FloatField(Field, DRFFloatField): pass


class HiddenField(Field, DRFHiddenField): pass


class HStoreField(Field, DRFHStoreField): pass


class IPAddressField(Field, DRFIPAddressField): pass


class ImageField(Field, DRFImageField): pass


class IntegerField(Field, DRFIntegerField): pass


class JSONField(Field, DRFJSONField): pass


class ListField(Field, DRFListField): pass


class ModelField(Field, DRFModelField): pass


class MultipleChoiceField(Field, DRFMultipleChoiceField): pass


class ReadOnlyField(Field, DRFReadOnlyField): pass


class RegexField(Field, DRFRegexField): pass


class SerializerMethodField(Field, DRFSerializerMethodField): pass


class SlugField(Field, DRFSlugField): pass


class TimeField(Field, DRFTimeField): pass


class URLField(Field, DRFURLField): pass


class UUIDField(Field, DRFUUIDField): pass



import pytest
from django.test import RequestFactory, TestCase
from django_filters import rest_framework as filters
from rest_framework import (
    serializers
)
from rest_framework.viewsets import ModelViewSet

from dseagull.djwt import JWTHS256
from dseagull.filters import BaseFilterSet
from dseagull.logger import LoggerMiddleware, thread_local
from tests.models.modeltest import Person


class PersonFilter(BaseFilterSet):
    last_name = filters.CharFilter()
    created = filters.CharFilter(method='filter_datetime', )

    class Meta:
        model = Person
        fields = ('id',)


class PersonSerializer(serializers.ModelSerializer):
    first_name = serializers.CharField()

    class Meta:
        model = Person
        fields = ('id', 'first_name',)


class PersonViewSet(ModelViewSet):
    serializer_class = PersonSerializer
    queryset = Person.objects.all()
    ordering_fields = ('first_name',)
    search_fields = ['first_name', ]
    filterset_class = PersonFilter


class TestFilter(TestCase):
    #
    def setUp(self):
        Person.objects.create(first_name='name1', last_name='last_name1')
        Person.objects.create(first_name='name2', last_name='last_name2')

    @pytest.mark.django_db
    def test_ordering(self):
        thread_local.remote_ip = 'remote_ip'
        view = PersonViewSet.as_view({'get': 'list'}, )
        factory = RequestFactory()
        factory.META = factory.GET = {'HTTP_AUTHORIZATION': f'Bearer x'}
        LoggerMiddleware(get_response=lambda x: None)(factory)  # noqa
        factory.META = factory.GET = {'HTTP_AUTHORIZATION': f'Bearer {JWTHS256().encode({})}'}
        LoggerMiddleware(get_response=lambda x: None)(factory)  # noqa
        request = factory.get('/?ordering=first_name', )
        response = view(request)
        assert response.data['results'] == [{'id': 1, 'first_name': 'name1'}, {'id': 2, 'first_name': 'name2'}]

        request = factory.get('/?ordering=-first_name', )
        response = view(request)
        assert response.data['results'] == [{'id': 2, 'first_name': 'name2'}, {'id': 1, 'first_name': 'name1'}]

    @pytest.mark.django_db
    def test_search(self):
        view = PersonViewSet.as_view({'get': 'list'})
        factory = RequestFactory()
        request = factory.get('/?search=name1', )
        response = view(request)
        assert response.data['results'] == [{'id': 1, 'first_name': 'name1'}, ]

        request = factory.get('/?search=name2', )
        response = view(request)
        assert response.data['results'] == [{'id': 2, 'first_name': 'name2'}, ]

    @pytest.mark.django_db
    def test_filter(self):
        view = PersonViewSet.as_view({'get': 'list'})
        factory = RequestFactory()
        request = factory.get('/?last_name=last_name1', )
        response = view(request)
        assert response.data['results'] == [{'id': 1, 'first_name': 'name1'}, ]

        request = factory.get('/?last_name=last_name2', )
        response = view(request)
        assert response.data['results'] == [{'id': 2, 'first_name': 'name2'}, ]

    @pytest.mark.django_db
    def test_base_filter(self):
        view = PersonViewSet.as_view({'get': 'list'})
        factory = RequestFactory()
        Person.objects.filter(id=1).update(created='2025-02-06 00:00')
        Person.objects.filter(id=2).update(created='2025-01-06 00:00')
        request = factory.get('/?created=1738771200,1738771201', )
        response = view(request)
        assert response.data['results'] == [{'id': 1, 'first_name': 'name1'}, ]
        request = factory.get('/?created=1735179200,1738771199', )
        response = view(request)
        assert response.data['results'] == [{'id': 2, 'first_name': 'name2'}, ]



from django.urls import include, path
from rest_framework.response import Response
from rest_framework.views import APIView


class SaeCheckPreloadView(APIView):
    def get(self, request):  # noqa
        return Response('successful')


def include_sae_urls():
    urls = [
        path('checkpreload', SaeCheckPreloadView.as_view(authentication_classes=[], permission_classes=[]), name='sae-checkpreload'),
    ]
    return include((urls, 'sae'), namespace='sae')



import datetime

from django.utils import timezone
from django_filters.rest_framework import FilterSet


class BaseFilterSet(FilterSet):

    def filter_datetime(self, queryset, name, value):  # noqa
        if ',' in value:
            start_at = timezone.make_aware(datetime.datetime.fromtimestamp(int(value.split(',')[0])))
            end_at = timezone.make_aware(datetime.datetime.fromtimestamp(int(value.split(',')[1])))
            query = {
                f'{name}__gte': start_at,
                f'{name}__lte': end_at,
            }
            queryset = queryset.filter(**query)
        return queryset



from django.db import models


class BaseModel(models.Model):
    class Meta:
        abstract = True

    created = models.DateTimeField(
        auto_now_add=True, verbose_name='创建时间', db_index=True,
    )
    updated = models.DateTimeField(
        auto_now=True, verbose_name='更新时间', db_index=True,
    )



from django.apps import apps
from django.conf import settings
from django.test import TestCase, override_settings

from dseagull.checks import jwt_check


class TestChecks(TestCase):

    @override_settings(JWT_KEY=None, JWT_EXP=None)
    def test_pagination_settings(self):
        self.assertIsNone(settings.JWT_KEY)
        self.assertIsNone(settings.JWT_EXP)

        errors = jwt_check(app_configs=None)
        errors = [error.msg for error in errors]
        self.assertEqual(4, len(errors))
        self.assertIn('请配置 jwt 的加密秘钥 JWT_KEY', errors[0])
        self.assertIn('请配置 jwt 的过期时间(单位秒) JWT_EXP', errors[1])
        self.assertIn('请为 LOGGING 添加配置 django.request', errors[2])
        self.assertIn('请配置 DJANGO_REQUEST_ERROR_WEBHOOK', errors[3])

    @override_settings(DJANGO_REQUEST_ERROR_WEBHOOK="test", LOGGING={'version': 1, "handlers": {}, 'loggers': {'django.request': {"handlers": []}}})
    def test_django_request_error_webhook(self):
        config = apps.get_app_config('dseagull')
        config.ready()
        errors = jwt_check(app_configs=None)
        errors = [error.msg for error in errors]
        self.assertEqual(0, len(errors), errors)
        self.assertIn("django.request", settings.LOGGING['loggers'])



from rest_framework.fields import ChoiceField  # noqa
from rest_framework.fields import DecimalField  # noqa
from rest_framework.fields import FileField  # noqa
from rest_framework.fields import FilePathField  # noqa
from rest_framework.fields import HiddenField  # noqa
from rest_framework.fields import ModelField  # noqa
from rest_framework.fields import MultipleChoiceField  # noqa
from rest_framework.fields import ReadOnlyField  # noqa
from rest_framework.fields import RegexField  # noqa
from rest_framework.fields import SerializerMethodField  # noqa

from dseagull.fields import BooleanField  # noqa
from dseagull.fields import CharField  # noqa
from dseagull.fields import DateField  # noqa
from dseagull.fields import DateTimeField  # noqa
from dseagull.fields import DictField  # noqa
from dseagull.fields import DurationField  # noqa
from dseagull.fields import EmailField  # noqa
from dseagull.fields import Field  # noqa
from dseagull.fields import FloatField  # noqa
from dseagull.fields import HStoreField  # noqa
from dseagull.fields import IPAddressField  # noqa
from dseagull.fields import ImageField  # noqa
from dseagull.fields import IntegerField  # noqa
from dseagull.fields import JSONField  # noqa
from dseagull.fields import ListField  # noqa
from dseagull.fields import SlugField  # noqa
from dseagull.fields import TimeField  # noqa
from dseagull.fields import URLField  # noqa
from dseagull.fields import UUIDField  # noqa



from types import SimpleNamespace

from django.utils import timezone
from django.utils.deprecation import MiddlewareMixin

from dseagull.dlogging import LOGGER


class BaseMiddleware(MiddlewareMixin):

    @classmethod
    def process_request(cls, request):
        request.start_at = timezone.localtime()
        try:
            body = request.body.decode('utf8')
            body = body[:1000]
            LOGGER.info(f'[request] method:{request.method};path={request.path}', )
            LOGGER.info(f"[request] body:{body}", )
        except:  # noqa
            pass

    def process_response(self, request, response):  # noqa

        response = response or SimpleNamespace(content=b'', status_code=201)
        try:
            response_content = response.content
        except:  # noqa
            return response
        if len(response_content) > 1000:
            response_content = response_content[:997] + b'...'
        try:
            response_content = response_content.decode('utf8')
        except:  # noqa
            pass

        duration = timezone.localtime() - request.start_at
        LOGGER.info(f'[process_response] status_code:{response.status_code};duration:{duration.seconds}.{duration.microseconds:0>6};content:{response_content}')
        if duration.seconds >= 10:
            LOGGER.info(f'[请求超时] method:{request.method};path={request.path};duration:{duration.seconds}.{duration.microseconds:0>6}', )
        return response



from rest_framework.request import Request
from rest_framework.test import APIRequestFactory

from dseagull.pagination import PageNumberPagination


class TestPageNumberPagination:

    def setup_method(self):
        class ExamplePagination(PageNumberPagination):
            page_size = 5

        self.pagination = ExamplePagination()
        self.queryset = range(1, 101)

    def paginate_queryset(self, request):
        return list(self.pagination.paginate_queryset(self.queryset, request))

    def get_paginated_content(self, queryset):
        response = self.pagination.get_paginated_response(queryset)
        return response.data

    def test_second_page(self):
        request = Request(APIRequestFactory().get('/', {'page': 2}))
        queryset = self.paginate_queryset(request)
        content = self.get_paginated_content(queryset)
        assert queryset == [6, 7, 8, 9, 10]
        assert content == {
            'paging': {'page_size': 5, 'skip': 5, 'total': 100, 'page': 2, 'total_pages': 20},
            'results': [6, 7, 8, 9, 10]
        }



import re
from pathlib import Path

from django.conf import settings
from django.core.management.base import BaseCommand


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('-n', '--name', )

    def handle(self, *_, **options):

        project_name = settings.BASE_DIR.parts[-1]
        name = options['name']
        lower_name = re.sub(r'(?<!^)(?=[A-Z])', '_', name).lower()

        # 添加 model 文件
        Path(f"{settings.BASE_DIR}/app/models").mkdir(exist_ok=True)
        Path(f"{settings.BASE_DIR}/app/models/__init__.py").touch(exist_ok=True)
        Path(f"{settings.BASE_DIR}/app/models/{lower_name}.py").touch(exist_ok=True)
        modelpy = (f"from django.db import models\n"
                   f"from dseagull.models import BaseModel\n\n\n"
                   f"class {name}(BaseModel):\n"
                   f"    pass\n")
        Path(f"{settings.BASE_DIR}/app/models/{lower_name}.py").write_text(modelpy)

        # 添加 serializer 文件
        Path(f"{settings.BASE_DIR}/app/serializers").mkdir(exist_ok=True)
        Path(f"{settings.BASE_DIR}/app/serializers/__init__.py").touch(exist_ok=True)
        Path(f"{settings.BASE_DIR}/app/serializers/{lower_name}.py").touch(exist_ok=True)
        serializerpy = (f"from rest_framework.serializers import ModelSerializer\n"
                        f"from dseagull.serializers import DateTimeField\n\n"
                        f"from app.models.{lower_name} import {name}\n\n\n"
                        f"class {name}CreateSerializer(ModelSerializer):\n"
                        f"    create_at = DateTimeField(help_text='创建时间', format='%Y-%m-%d %H:%M:%S')  # todo\n\n"
                        f"    class Meta:\n"
                        f"        model = {name}\n"
                        f"        fields = (\n"
                        f"            'id', 'create_at',\n"
                        f"        )\n\n\n"
                        f"class {name}ListSerializer(ModelSerializer):\n"
                        f"    class Meta:\n"
                        f"        model = {name}\n"
                        f"        fields = (\n"
                        f"            'id',\n"
                        f"        )\n")
        Path(f"{settings.BASE_DIR}/app/serializers/{lower_name}.py").write_text(serializerpy)

        # 创建 view 文件
        Path(f"{settings.BASE_DIR}/app/views").mkdir(exist_ok=True)
        Path(f"{settings.BASE_DIR}/app/views/__init__.py").touch(exist_ok=True)
        Path(f"{settings.BASE_DIR}/app/views/{lower_name}.py").touch(exist_ok=True)
        viewpy = (f"from django_filters import rest_framework as filters\n"
                  f"from rest_framework.decorators import action\n"
                  f"from rest_framework.versioning import URLPathVersioning\n"
                  f"from rest_framework.viewsets import ModelViewSet\n\n"
                  f"from app.models.{lower_name} import {name}\n"
                  f"from app.serializers.{lower_name} import {name}CreateSerializer\n"
                  f"from app.serializers.{lower_name} import {name}ListSerializer\n"
                  f"from dseagull.filters import BaseFilterSet\n\n\n"
                  f"class {name}Filter(BaseFilterSet):\n"
                  f"    create_at = filters.CharFilter(help_text='创建时间', method='filter_datetime', )\n\n"
                  f"    class Meta:\n"
                  f"        model = {name}\n"
                  f"        fields = ('id',)\n\n\n"
                  f"class {name}ViewSet(ModelViewSet):\n"
                  f"    serializer_class = {name}CreateSerializer\n"
                  f"    queryset = {name}.objects.all()\n"
                  f"    filterset_class = {name}Filter\n"
                  f"    versioning_class = URLPathVersioning\n\n"
                  f"    def get_queryset(self):  # todo 请实现数据隔离\n"
                  f"        return super().get_queryset()\n\n"
                  f"    @action(methods=['get'], detail=False, serializer_class={name}ListSerializer, )\n"
                  f"    def paginator_list(self, request, *args, **kwargs):\n"
                  f"        return super().list(request=request, *args, **kwargs)\n")
        Path(f"{settings.BASE_DIR}/app/views/{lower_name}.py").write_text(viewpy)

        # 创建 routers 文件
        routers_path = Path(f"{settings.BASE_DIR}/{project_name}/routers.py")
        if not routers_path.exists():
            routers_path.touch()
            routerspy = ("from rest_framework import routers\n\n"
                         "router = routers.DefaultRouter()\n"
                         "router.trailing_slash = '/?'\n"
                         # f"router.routes[2].mapping['put'] = router.routes[2].mapping.pop('patch', None)\n"
                         "v1 = '^(?P<version>(1))'\n")
            routers_path.write_text(routerspy)
        write_text = read_text = routers_path.read_text()

        router_start_text = f'from app.views.{lower_name} import {name}ViewSet'
        if router_start_text not in read_text:
            write_text = f'{router_start_text}\n{write_text}'

        router_end_text = "router.register(f'{v1}/" + lower_name + f"s', {name}ViewSet, )"
        if router_end_text not in read_text:
            write_text = f'{write_text}\n{router_end_text}'

        routers_path.write_text(write_text)

        # 创建 testcase 文件
        Path(f"{settings.BASE_DIR}/app/tests").mkdir(exist_ok=True)
        Path(f"{settings.BASE_DIR}/app/tests/__init__.py").touch(exist_ok=True)
        Path(f"{settings.BASE_DIR}/app/tests/{lower_name}.py").touch(exist_ok=True)
        testcasepy = (f"import dseagull\n\n"
                      f"dseagull.django.setup('{project_name}.settings') # noqa\n\n"
                      f"from django.test.testcases import TestCase\n"
                      f"from rest_framework.test import APIClient\n\n\n"
                      f"class {name}TestCase(TestCase):\n"
                      f"    def test_cru(self):\n"
                      f"        client = APIClient()\n"
                      f"        r = client.post('/1/{lower_name}s/')\n"
                      f"        self.assertEqual(r.status_code, 201, r.data)\n"
                      f"        r = client.get('/1/{lower_name}s/')\n"
                      f"        self.assertEqual(r.status_code, 200, r.data)\n"
                      f"        self.assertEqual(r.data['paging']['total'], 1, r.data)\n")
        Path(f"{settings.BASE_DIR}/app/tests/{lower_name}.py").write_text(testcasepy)
        tests_path = Path(f"{settings.BASE_DIR}/app/tests/tests.py")
        tests_path.touch(exist_ok=True)
        read_text = tests_path.read_text()
        add_read_text = f"from app.tests.{lower_name} import {name}TestCase  # noqa"
        if add_read_text not in read_text:
            tests_path.write_text(f'{add_read_text}\n{read_text}')



from rest_framework import pagination
from rest_framework.response import Response


class PageNumberPagination(pagination.PageNumberPagination):
    max_page_size = 100
    page_size_query_param = 'page_size'
    page_query_description = '页数'
    page_size_query_description = '每页限定行数'
    invalid_page_message = '已经没有更多了'

    def get_paginated_response(self, data):
        page_size = self.get_page_size(self.request)
        return Response({
            'paging': {
                'page_size': page_size,
                'skip': (self.page.number - 1) * page_size,
                'total': self.page.paginator.count,
                'page': self.page.number,
                'total_pages': self.page.paginator.num_pages,
            },
            'results': data
        })



import json
import uuid

from django.conf import settings
from django.core.checks import Tags, register, Critical


@register(Tags.compatibility)
def jwt_check(app_configs, **kwargs) -> list:  # noqa
    errors = []
    if not hasattr(settings, 'JWT_KEY') or not settings.JWT_KEY:
        errors.append(
            Critical(
                f"请配置 jwt 的加密秘钥 JWT_KEY, 比如: JWT_KEY = '{uuid.uuid4().hex}{uuid.uuid4().hex}'"
            )
        )

    if not hasattr(settings, 'JWT_EXP') or not settings.JWT_EXP:
        errors.append(
            Critical(
                "请配置 jwt 的过期时间(单位秒) JWT_EXP, 比如: JWT_EXP = 60 * 60 * 24 * 30"
            )
        )

    logging = getattr(settings, 'LOGGING', {})
    logger = logging.get('loggers', {}).get('django.request')
    if not logger:
        conf = {
            'version': 1,
            'formatters': {
                'django.request': {
                    'format': '[%(levelname)1.1s %(asctime)s %(module)s.%(funcName)s:%(lineno)d];%(message)s'
                },
            },
            'handlers': {
                'webhook': {'level': 'ERROR', 'class': 'dseagull.dlogging.DjangoRequestErrorLOGGINGHandler'},
                'console': {'level': 'INFO', 'class': 'logging.StreamHandler', 'formatter': 'django.request', },
            },
            'loggers': {
                'django.request': {'handlers': ['console', 'webhook'], 'level': 'INFO', 'propagate': False, 'encoding': 'utf8'}
            }
        }
        errors.append(
            Critical(
                f"请为 LOGGING 添加配置 django.request: \n{json.dumps(conf, indent=4)}"
            )
        )

        webhook = getattr(settings, 'DJANGO_REQUEST_ERROR_WEBHOOK', None)
        if webhook is None:
            errors.append(
                Critical(
                    f"请配置 DJANGO_REQUEST_ERROR_WEBHOOK, 目前只支持钉钉机器人的 webhook, 配置文档: https://open.dingtalk.com/document/robots/custom-robot-access, 安全设置->自定义关键词填入 Seagull"
                )
            )

    return errors



# todo
# import logging
import threading
from uuid import uuid4

#
from django.utils.deprecation import MiddlewareMixin

from dseagull.djwt import JWTHS256

#
# # pip install python-json-logger
# from pythonjsonlogger import jsonlogger
# # 创建一个全局线程局部存储，用于保存请求 ID
thread_local = threading.local()


#
#
# class RequestIDFilter(logging.Filter):
#     def filter(self, record):
#         record.__dict__.update(vars(thread_local))
#         return True
#
#
# # 配置日志
# logger = logging.getLogger(__name__)
# logger.setLevel(logging.INFO)
# handler = logging.StreamHandler()
# # formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(request_id)s - %(message)s')
# formatter = jsonlogger.JsonFormatter('%(asctime)s %(user_id)s %(levelname)s %(request_id)s %(message)s')
#
# handler.setFormatter(formatter)
# logger.addHandler(handler)
#
# # 添加自定义的 Filter
# logger.addFilter(RequestIDFilter())
#
#
# # 日志：
# # 服务端日志输出格式通常会根据具体的应用场景和需求而有所不同，但一般来说，大厂的服务端日志输出会遵循一定的结构化格式，以便于后续的日志分析和监控。常见的日志格式包括 JSON、文本格式、Apache 日志格式等。
# #
# # 以下是一个常见的服务端日志输出样例，采用 JSON 格式：
# #
# # json
# # {
# #   "timestamp": "2023-10-01T12:34:56Z",
# #   "level": "INFO",
# #   "service": "user-service",
# #   "instance": "instance-1",
# #   "requestId": "abc123",
# #   "userId": "user-456",
# #   "userType": "user-456",
# #   "action": "login",
# #   "status": "success",
# #   "responseTime": 123,
# #   "ip": "192.168.1.1",
# #   "userAgent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36",
# #   "message": "User logged in successfully"
# # }
# # 在这个示例中，包含了以下字段：
# #
# # timestamp: 日志生成的时间。
# # level: 日志级别（如 INFO、ERROR、DEBUG 等）。
# # service: 产生日志的服务名称。
# # instance: 服务实例标识。
# # requestId: 唯一的请求标识符，方便追踪请求。
# # userId: 用户标识。
# # userType: user-456,
# # action: 用户执行的操作。
# # status: 操作的状态（成功或失败）。
# # responseTime: 处理请求所需的时间（毫秒）。
# # ip: 用户的 IP 地址。
# # userAgent: 用户的浏览器信息。
# # message: 具体的日志信息描述。
# # 这种结构化的日志格式可以方便地进行搜索、过滤和分析，适合大规模的分布式系统和微服务架构。大厂通常会使用 ELK（Elasticsearch, Logstash, Kibana）等工具来集中管理和分析日志。

class LoggerMiddleware(MiddlewareMixin):

    @classmethod
    def process_request(cls, request):
        authorization = request.META.get('HTTP_AUTHORIZATION') or request.GET.get('Authorization') or ''
        if authorization:
            request.META['HTTP_AUTHORIZATION'] = authorization

        # 先清空之前的值
        for attr in list(thread_local.__dict__.keys()):
            delattr(thread_local, attr)

        if authorization.startswith('Bearer '):
            try:
                payload = JWTHS256().decode(authorization.removeprefix('Bearer '))
            except:  # noqa
                return

            thread_local.userId = payload.get('userId')
            thread_local.userType = payload.get('userType')

        thread_local.remote_ip = request.META.get('HTTP_X_REAL_IP') or request.META.get('HTTP_X_FORWARDED_FOR') or request.META.get('REMOTE_ADDR') or ''
        thread_local.remote_ip = thread_local.remote_ip.split(',')[0]
        thread_local.request_id = request.META.get('HTTP_X_REQUEST_ID', uuid4().hex)

    def process_response(self, request, response):  # noqa
        return response
